#!/bin/bash
#
# build_gsl.sh <product_dir> <debug|opt|prof> [tar] [source]
# (in source build required)

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

function ncores()
{
    local result
    (( result = 0 ))
    case `uname -s` in
        Darwin)
            (( result = `sysctl -n hw.ncpu` ))
            ;;
        Linux)
            (( result = `cat /proc/cpuinfo 2>/dev/null | grep -c -e '^processor'` ))
            ;;
        *)
            echo "Unknown OS: `uname -s`" 1>&2
    esac
    printf "$result\n"
    return $(( result == 0 ))
}

function die()
{
  echo "$1"; exit 2
}

get_this_dir()
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

make_tarball()
{
    echo ""
    set -x
    cd ${product_dir}
    tarname=${package}-${pkgdotver}-${OS}-${plat}-${qualdir}.tar.bz2
    tar -cjf ${tarname} \
       ${package}/${pkgver}.version/${flvr}_${qualver} \
       ${package}/${pkgver}/ups \
       ${package}/${pkgver}/source \
       ${package}/${pkgver}/${flvr}-${qualdir}
    cd ${thisdir}
    set +x
    exit 0
}

make_source_tarball()
{
    set -x
    cd ${product_dir}
    tar cjf ${package}-${pkgdotver}-source.tar.bz2 \
       ${package}/${pkgver}/build_${package}.sh \
       ${package}/${pkgver}/ups \
       ${package}/${pkgver}/tar
    cd ${thisdir}
    set +x
}

set_qualifiers()
{
    if [ -z ${extraqual} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${extraqual}
    qualdir=${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	      qualver=${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	      qualver=${extraqual}
    elif [ "${extraqual}" = "prof" ]
    then
	      qualver=${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

check_with_lsb_release ()
{
   plat=`uname -p`
   if [ `lsb_release -d | cut -f2 | cut  -f1 -d" "` = "Ubuntu" ]
   then
      OS1=ub
      OSnum=`lsb_release -r | cut -f2 | cut -f1 -d"."`
      OS=${OS1}${OSnum}
      plat=`uname -m`
   elif [ ${OS1} = "Linux" ]
   then
      OSnum=`lsb_release -r | cut -f2 | cut -f1 -d"."`
      OS=${OS1}${OSnum}
      # Scientific Linux - slf should work
      if [ `lsb_release -d | cut  -f3 -d" "` = "SL" ]
      then
	 OS=slf${OSnum}
      # Scientific Linux Fermi
      elif [ `lsb_release -d | cut  -f3 -d" "` = "SLF" ]
      then
	 OS=slf${OSnum}
      #
      elif [ `lsb_release -i | cut -f2` = "ScientificFermi" ]
      then
	 OS=slf${OSnum}
      # pretend that SL6 is the same as SLF6
      elif [ `lsb_release -i | cut -f2` = "Scientific" ]
      then
	 OS=slf${OSnum}
      # pretend that CentOS is SLF
      elif [ `lsb_release -i | cut -f2` = "CentOS" ]
      then
	 OS=slf${OSnum}
      # pretend that RedHatEnterpriseServer is SLF
      elif [ `lsb_release -i | cut -f2` = "RedHatEnterpriseServer" ]
      then
	 OS=slf${OSnum}
      # Scientific Linux CERN
      elif [ `lsb_release -d | cut  -f4 -d" "` = "SLC" ]
      then
	 OS=slc${OSnum}
      elif [ `lsb_release -d | cut  -f4 -d" "` = "LTS" ]
      then
	 OS=slf${OSnum}
      # unrecognized - pretend that this is SLF
      else
	 OS=slf${OSnum}
      fi
   fi
}

default_names ()
{
      OSnum1=`uname -r | cut -f1 -d"."`
      OSnum2=`uname -r | cut -f2 -d"."`
      OS=${OS1}${OSnum1}${OSnum2}
      plat=`uname -p`
}

check_linux ()
{
   plat=`uname -p`
   if [ -e /etc/system-release-cpe ]
   then
      OSnum=`cat /etc/system-release-cpe | cut -f5 -d":" |  cut -f1 -d"."`
      if [ `cat /etc/redhat-release | cut  -f4 -d" "` = "SLC" ]
      then
         OS=slc${OSnum}
      else
         OS=slf${OSnum}
      fi
   elif [ -e /etc/redhat-release ]
   then
      if [ `cat /etc/redhat-release | cut  -f4 -d" "` = "SLC" ]
      then
         OS=slc${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f6 -d" " |  cut -f1 -d"."`
      elif [ `cat /etc/redhat-release | cut  -f3 -d" "` = "Fermi" ]
      then
         OS=slf${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f5 -d" " |  cut -f1 -d"."`
      elif [ `cat /etc/redhat-release | cut  -f3 -d" "` = "SLF" ]
      then
         OS=slf${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f5 -d" " |  cut -f1 -d"."`
      else
         OS=slf${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f5 -d" " |  cut -f1 -d"."`
      fi
   else
      default_names
   fi
}

get_platform_name()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`

    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
	plat=`uname -m`
	OSnum=`uname -r | cut -f1 -d"."`
	macos1=`sw_vers -productVersion | cut -d'.' -f1`
	macos2=`sw_vers -productVersion | cut -d'.' -f2`
	macos=${macos1}.${macos2}
	OS=osx${macos1}${macos2}
    elif [ "${flvr0}" = "Linuxppc" ]
    then
       OS=slf5
       plat=`uname -p`
    elif [ -e /usr/bin/lsb_release ]
    then
       check_with_lsb_release
    elif [ ${OS1} = "Linux" ]
    then
       check_linux
    else
       default_names
    fi

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi
    
    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    sourcedir=${product_dir}/${package}/${pkgver}/source
    tardir=${product_dir}/${package}/${pkgver}/tar
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
}

declare_product()
{
    echo "Declaring ${package} ${pkgver} for ${flvr} and ${fullqual}"
    if [ ${OS1} = "Darwin" ]
    then
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -2 -m ${package}.table -q ${fullqual} -z ${product_dir}
    else
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -4 -m ${package}.table -q ${fullqual} -z ${product_dir}
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
extraqual=${2}
maketar=${3}
sourcetar=${4}

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=gsl
pkgver=v1_15
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtar=gsl-${pkgdotver}.tar.gz

get_this_dir

if [ "${sourcetar}" = "source" ]
then
  make_source_tarball
fi

set_qualifiers

get_platform_name


if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   make_tarball
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    cxxflg="-O3 -g -DNDEBUG"
elif [ "${extraqual}" = "debug" ]
then
    cxxflg="-g -O0"
elif [ "${extraqual}" = "prof" ]
then
    cxxflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
fi

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

set -x

cd ${sourcedir}
echo "In code directory: ${PWD}"
tar xf ${tardir}/${pkgtar} || die "Unable to undwind ${tardir}/${pkgtar}"

cd ${blddir} || die "Unable to cd to ${blddir}"
config_dir=${sourcedir}/gsl-${pkgdotver}
echo "config_dir is: ${config_dir}"
if [ ! -f ${config_dir}/configure ]; then
  echo "There is no configure script in ${config_dir}"
  exit 1
fi
CFLAGS="${cxxflg}" ${config_dir}/configure --prefix=${pkgdir} --enable-shared --enable-static=no --with-pic
make -j `ncores` || die "Failed while building gsl"
if [ ${OS1} = "Darwin" ]
then
  echo "ignore broken make check on ${OS1}"
else
  make -j `ncores` check || die "Failed running make check"
fi
make install || die "Failed while installing gsl"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

declare_product

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "${package} is installed at ${GSL_FQ_DIR}"


# need to fix paths in bin/gsl-config

rm -f /tmp/gslfix.sh

echo "#!/bin/bash" > /tmp/gslfix.sh
echo "" >>  /tmp/gslfix.sh
echo "mv ${GSL_FQ_DIR}/bin/gsl-config ${GSL_FQ_DIR}/bin/gsl-config.bak" >> /tmp/gslfix.sh
echo "cat ${GSL_FQ_DIR}/bin/gsl-config.bak | sed 's%${GSL_LIB}%\${GSL_LIB}%'  | sed 's%${GSL_INC}%\${GSL_INC}%'   | sed 's%${GSL_FQ_DIR}%\${GSL_FQ_DIR}%'> ${GSL_FQ_DIR}/bin/gsl-config" >> /tmp/gslfix.sh
echo "chmod +x  ${GSL_FQ_DIR}/bin/gsl-config" >> /tmp/gslfix.sh
echo "" >>  /tmp/gslfix.sh

chmod +x /tmp/gslfix.sh
/tmp/gslfix.sh



# this must be last because it will exit
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   make_tarball
fi

exit 0
